# Copyright 2015 by Toben "Narcolapser" Archer. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its documentation for any purpose 
# and without fee is hereby granted, provided that the above copyright notice appear in all copies and 
# that both that copyright notice and this permission notice appear in supporting documentation, and 
# that the name of Toben Archer not be used in advertising or publicity pertaining to distribution of 
# the software without specific, written prior permission. TOBEN ARCHER DISCLAIMS ALL WARRANTIES WITH 
# REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT 
# SHALL TOBEN ARCHER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
# OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import logging
import json
import requests
import time

logging.basicConfig(filename='o365.log',level=logging.DEBUG)

log = logging.getLogger(__name__)

class Event( object ):
	#Formated time string for translation to and from json.
	time_string = '%Y-%m-%dT%H:%M:%SZ'
	#takes a calendar ID
	create_url = 'https://outlook.office365.com/api/v1.0/me/calendars/{0}/events'
	#takes current event ID
	update_url = 'https://outlook.office365.com/api/v1.0/me/events/{0}'
	#takes current event ID
	delete_url = 'https://outlook.office365.com/api/v1.0/me/events/{0}'


	def __init__(self,json=None,auth=None,cal=None):
		self.auth = auth
		self.calendar = cal
		self.attendees = []

		if json:
			self.json = json
			self.isNew = False
		else:
			self.json = {}


	def create(self,calendar=None):
		'''
		this method creates an event on the calender passed.
		IMPORTANT: It returns that event now created in the calendar, if you wish
		to make any changes to this event after you make it, use the returned value
		and not this particular event any further.
		'''
		if not self.auth:
			return False

		if calendar:
			calId = calendar.calendarId
			self.calendar = calendar
		elif self.calendar:
			calId = self.calendar.calendarId
		else:
			return False

		headers = {'Content-type': 'application/json', 'Accept': 'application/json'}

		try:
			self.json['Attendees'] = self.attendees
		except:
			return False

		log.debug('creating json for request.')
		data = json.dumps(req)

		try:
			log.debug('sending post request now')
			response = requests.post(self.create_url.format(calId),data,headers=headers,auth=self.auth)
		except:
			log.debug('response to event creation: %s',str(response))
			return False

		log.debug('response to event creation: %s',str(response))
		return Event(response.json(),self.auth,calendar)

	def update(self,calendar=None):
		'''
		This method updates an event that already exists in a calendar. It simply 
		re-uploads the local json, so change things before you call this function.
		'''
                if not self.auth:
                        return False

                if calendar:
                        calId = calendar.calendarId
                        self.calendar = calendar
                elif self.calendar:
                        calId = self.calendar.calendarId
                else:
                        return False


                headers = {'Content-type': 'application/json', 'Accept': 'application/json'}

                try:
                        req = {}
                        req['Subject'] = self.subject
                        req['Body'] = {'ContentType':'HTML','Content':self.body}
                        req['Start'] = time.strftime(self.time_string,self.start)
                        req['End'] = time.strftime(self.time_string,self.end)
                        req['Attendees'] = self.attendees
                except:
                        return False

                log.debug('creating json for request.')
                data = json.dumps(req)

		try:
			response = requests.patch(self.update_url.format(self.Id),data,headers=headers,auth=self.auth)
			log.debug('sending patch request now')
		except:
			log.debug('response to event creation: %s',str(response))
			return False

		log.debug('response to event creation: %s',str(response))

		return Event(response.json(),self.auth)		


	def delete(self):
		'''
		delete's an event from the calendar it is in. But leaves you this handle.
		You could, in theory, then change the calendar and transfer the event to
		that new calendar. You know, if that's your thing.
		'''
		if not self.auth:
			return False
		if not self.Id:
			return False

		headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}

		try:
			log.debug('sending delete request')
			response = requests.delete(self.delete_url.format(self.Id),headers=headers,auth=self.auth)
		except:
			return False
		finally:
			log.debug('response to deletion: %s',str(response))

		return response

	def toJson(self):
		'''
		Creates a JSON representation of the calendar event! oh. uh. I mean it
		simply returns the json representation that has always been in self.json.
		'''
		return self.json

	def fullcalendarioJson(self):
		'''
		returns a form of the event suitable for the vehicle booking system here.
		oh the joys of having a library to yourself! 
		'''
		ret = {}
		ret['title'] = self.json['Subject']
		ret['driver'] = self.json['Organizer']['EmailAddress']['Name']
		ret['driverEmail'] = self.json['Organizer']['EmailAddress']['Address']
		ret['start'] = self.json['Start']
		ret['end'] = self.json['End']
		ret['IsAllDay'] = self.json['IsAllDay']
		return ret

	def getSubject(self):
		return self.json['Subject']

	def getBody(self):
		return self.json['Body']['Content']

	def getStart(self):
		return time.strptime(self.json['Start'], self.time_string)

	def getEnd(self):
		return time.strptime(self.json['End'], self.time_string)

	def getAttendees(self):
		return self.json['Attendees']

	def addAttendee(self,val):
		self.json['Attendees'].append(val)

	def setSubject(self,val):
		self.json['Subject'] = val

	def setBody(self,val):
		self.json['Body']['Content'] = val

	def setStart(self,val):
		self.json['Start'] = time.strftime(self.time_string,val)

	def setEnd(self,val):
		self.json['End'] = time.strftime(self.time_string,val)

	def setAttendees(self,val):
		self.json['Attendees'] = val

#To the King!



# Copyright 2015 by Toben "Narcolapser" Archer. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its documentation for any purpose 
# and without fee is hereby granted, provided that the above copyright notice appear in all copies and 
# that both that copyright notice and this permission notice appear in supporting documentation, and 
# that the name of Toben Archer not be used in advertising or publicity pertaining to distribution of 
# the software without specific, written prior permission. TOBEN ARCHER DISCLAIMS ALL WARRANTIES WITH 
# REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT 
# SHALL TOBEN ARCHER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
# OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""
This file contains the functions for working with attachments. Including the ability to work with the
binary of the file directly. The file is stored locally as a string using base64 encoding. 
"""

import base64
import logging
import json
import requests

logging.basicConfig(filename='o365.log',level=logging.DEBUG)

log = logging.getLogger(__name__)

class Attachment( object ):
	'''
	Attachment class is the object for dealing with attachments in your messages. To add one to
	a message, simply append it to the message's attachment list (message.attachments). 
	'''
	def __init__(self,json=None):
		if json:
			self.json = json
			self.isPDF = '.pdf' in self.json['Name'].lower()
		else:
			self.json = {}

	def isType(self,typeString):
		'''
		This function lets you know what type the file is.
		'''
		return '.'+typeString.lower() in self.json['Name'].lower()

	def getType(self):
		'''
		returns the file extension
		'''
		return self.json['Name'][self.json['Name'].rindex('.'):]

	def save(self,location):
		'''
		Location: path to where the file is to be saved.

		Save the attachment locally to disk.
		'''
		try:
			outs = open(location+'/'+self.Name,'wb')
			outs.write(base64.b64decode(self.json['ContentBytes']))
			outs.close()
			log.debug('file saved locally.')
			
		except Exception as e:
			log.debug('file failed to be saved: %s',str(e))
			return False

		log.debug('file saving successful')
		return True

	def getByteString(self):
		'''
		fetch the binary representation of the file. useful for times you want to
		skip the step of saving before sending it to another program. This allows
		you to make scripts that use linux pipe lines in their execution.
		'''
		try:
			return base64.b64decode(self.json['ContentBytes'])

		except Exception as e:
			log.debug('what? no clue what went wrong here. cannot decode attachment.')

		return False

	def getBase64(self):
		'''
		fetches the base64 encoding representation of the attachment.
		'''
		try:
			return self.json['ContentBytes']
		except Exception as e:
			log.debug('what? no clue what went wrong here. probably no attachment.')
		return False

	def setByteString(self,val):
		'''
		sets the file for this attachment from a byte string.
		'''
		try:
			self.json['ContentBytes'] = base64.b64encode(val)
		except:
			log.debug('error encoding attachment.')
			return False
		return True

	def setBase64(self,val):
		'''
		Sets the file for this attachment from a base64 encoding.
		'''
		self.json['ContentBytes'] = val
		return true

#To the King!



# Copyright 2015 by Toben "Narcolapser" Archer. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its documentation for any purpose 
# and without fee is hereby granted, provided that the above copyright notice appear in all copies and 
# that both that copyright notice and this permission notice appear in supporting documentation, and 
# that the name of Toben Archer not be used in advertising or publicity pertaining to distribution of 
# the software without specific, written prior permission. TOBEN ARCHER DISCLAIMS ALL WARRANTIES WITH 
# REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT 
# SHALL TOBEN ARCHER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
# OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import requests
import base64
import json
import logging
import time

from O365.event import Event

logging.basicConfig(filename='o365.log',level=logging.DEBUG)

log = logging.getLogger(__name__)

class Calendar( object ):
	events_url = 'https://outlook.office365.com/api/v1.0/me/calendars/{0}/calendarview?startDateTime={1}&endDateTime={2}'
	time_string = '%Y-%m-%dT%H:%M:%SZ'

	def __init__(self, json=None, auth=None):
		'''
		Wraps all the informaiton for managing calendars.
		'''
		self.json = json
		self.auth = auth
		self.events = []

		if json:
			log.debug('translating calendar information into local variables.')
			self.calendarId = json['Id']
			self.name = json['Name']

	def getName(self):
		return self.json['Name']

	def getCalendarId(self):
		return self.calendarId['Id']

	def getId(self):
		return self.getCalendarId()

	def fetchEvents(self,start=None,end=None):
		'''
		So I originally made this function "fetchEvents" which was a terrible idea. Everything else is "getX" except
		events which were appearenty to good for that. So this function is just a pass through for legacy sake.
		'''
		return self.getEvents(start,end)


	def getEvents(self,start=None,end=None):
		'''
		Pulls events in for this calendar. default range is today to a year now.
		'''

		#If no start time has been supplied, it is assumed you want to start as of now.
		if not start:
			start = time.strftime(self.time_string)

		#If no end time has been supplied, it is assumed you want the end time to be a year
		#from what ever the start date was. 
		if not end:
			end = time.time()
			end += 3600*24*365.25
			end = time.gmtime(end)
			end = time.strftime(self.time_string,end)

		#This is where the actual call to Office365 happens.
		response = requests.get(self.events_url.format(self.json['Id'],start,end),auth=self.auth)
		log.info('Response from O365: %s', str(response))
		
		#This takes that response and then parses it into individual calendar events.
		for event in response.json()['value']:
			try:
				self.events.append(Event(event,self.auth,self))
				log.debug('appended event: %s',event['Subject'])
			except Exception as e:
				log.info('failed to append calendar: %',str(e))
		
		log.debug('all events retrieved and put in to the list.')
		return True

#To the King!



# Copyright 2015 by Toben "Narcolapser" Archer. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its documentation for any purpose 
# and without fee is hereby granted, provided that the above copyright notice appear in all copies and 
# that both that copyright notice and this permission notice appear in supporting documentation, and 
# that the name of Toben Archer not be used in advertising or publicity pertaining to distribution of 
# the software without specific, written prior permission. TOBEN ARCHER DISCLAIMS ALL WARRANTIES WITH 
# REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT 
# SHALL TOBEN ARCHER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
# OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

from O365 import Attachment
import logging
import json
import requests

logging.basicConfig(filename='o365.log',level=logging.DEBUG)

log = logging.getLogger(__name__)

class Message( object ):
	att_url = 'https://outlook.office365.com/api/v1.0/me/messages/{0}/attachments'
	send_url = 'https://outlook.office365.com/api/v1.0/me/sendmail'
	update_url = 'https://outlook.office365.com/api/v1.0/me/messages/{0}'

	def __init__(self, json=None, auth=None):
		'''
		Wraps all the informaiton for receiving messages.
		'''
		if json:
			self.json = json
			self.hasAttachments = json['HasAttachments']

		else:
			self.json = {}
			self.hasAttachments = False
	
		self.auth = auth
		self.attachments = []
		self.reciever = None


	def fetchAttachments(self):
		if not self.hasAttachments:
			log.debug('message has no attachments, skipping out early.')
			return False

		response = requests.get(self.att_url.format(self.json['Id']),auth=self.auth)
		log.info('response from O365 for retriving message attachments: %s',str(response))
		json = response.json()

		for att in json['value']:
			try:
				self.attachments.append(Attachment(att))
				log.debug('successfully downloaded attachment for: %s.',self.auth[0])
			except Exception as e:
				log.info('failed to download attachment for: %s', self.auth[0])

		return len(self.attachments)

	def sendMessage(self):
		if not self.receiver:
			return False

		headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
		message = {}
		message['Subject'] = self.subject
		message['Body'] = {'ContentType':'Text','Content':self.body}
		message['ToRecipients'] = [{'EmailAddress':{'Address':self.receiver}}]

		dat = {'Message':message,'SaveToSentItems':'true'}

		data = json.dumps(dat)
#		data = json.dumps(self.json)
		print data

		response = requests.post(self.send_url,data,headers=headers,auth=self.auth)
		print response

		return True
		
	def markAsRead(self):
		read = '{"IsRead":true}'
		headers = {'Content-type': 'application/json', 'Accept': 'application/json'}
		try:
			response = requests.patch(self.update_url.format(self.json['Id']),read,headers=headers,auth=self.auth)
		except:
			return False
		print response
		return True


	def getSender(self):
		return self.json['Sender']

	def getSenderEmail(self):
		return self.json['Sender']['EmailAddress']['Address']
	
	def getSenderName(self):
		return self.json['Sender']['EmailAddress']['Name']

	def getSubject(self):
		return self.json['Subject']

	def getBody(self):
		return self.json['Body']['Content']

	def setRecipients(self,val):
		if isinstance(val,list):
			self.json['ToRecipients'] = val
		elif isinstances(val,dict):
			self.json['ToRecipients'] = [val]
		elif isinstance(val,string):
			if '@' in val:
				self.json['ToRecipients'] = []
				self.addRecipient(None,val)
		else:
			return False
		return True

	def addRecipient(self,name,address):
		self.json['ToRecipients'].append({'EmailAddress':{'Address':address,'Name':name}})

	def setSubject(self,val):
		self.json['Subject']

	def setBody(self,val):
		self.json['Body']['Content']

#To the King!



#!/usr/bin/env python

from distutils.core import setup
setup(name='O365',
	version='0.5',
	description='Python library for working with Microsoft Office 365',
	author='Toben Archer',
	author_email='sandslash+O365@gmail.com',
	url='https://github.com/Narcolapser/python-o365',
	packages=['O365']
	)




# Copyright 2015 by Toben "Narcolapser" Archer. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its documentation for any purpose 
# and without fee is hereby granted, provided that the above copyright notice appear in all copies and 
# that both that copyright notice and this permission notice appear in supporting documentation, and 
# that the name of Toben Archer not be used in advertising or publicity pertaining to distribution of 
# the software without specific, written prior permission. TOBEN ARCHER DISCLAIMS ALL WARRANTIES WITH 
# REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT 
# SHALL TOBEN ARCHER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
# OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

from O365.message import Message
import logging
import json
import requests

logging.basicConfig(filename='o365.log',level=logging.DEBUG)

log = logging.getLogger(__name__)

class Inbox( object ):
	#url for fetching emails. Takes a flag for whether they are read or not.
	inbox_url = 'https://outlook.office365.com/api/v1.0/me/messages?$filter=IsRead eq {0}'

	def __init__(self, email, password,getNow=True):
		log.debug('creating inbox for the email %s',email)
		self.auth = (email,password)
		self.messages = []
		if getNow:
			self.getMessages()


	def getMessages(self,IsRead=False):
		'''
		You create an inbox to be the container class for messages, this method
		then pulls those messages down to the local disk. This is called in the
		init method, so it's kind of pointless for you. Unless you think new
		messages have come in.

		IsRead: Set this as True if you want to include messages that have been read.
		'''

		log.debug('fetching messages.')
		response = requests.get(self.inbox_url.format(str(IsRead).lower()),auth=self.auth)
		log.info('Response from O365: %s', str(response))
		
		for message in response.json()['value']:
			try:
				self.messages.append(Message(message,self.auth))
				log.debug('appended message: %s',message['Subject'])
			except Exception as e:
				log.info('failed to append message: %',str(e))
		
		log.debug('all messages retrieved and put in to the list.')
		return True

#To the King!
